#include "WR_Connection.h"
#include "WR_Submit.h"
#include "WR_Bind.h"
#include "SmppError.h"
#include "SmppBindResp.h"
#include "PA_ReceiveThreadPool.h"
#include "PA_SendThreadPool.h"
#include "PA_LongRangeDialogIdManager.h"
#include "Colors.h"

	WR_Connection::WR_Connection()
: PA_SmppConnection ( PCM_CALL_PDU_PROCESS )
{
	SimpleManagedObject* object;

	// The PA_SmppConnection connection contains more manageable components which are 
	// initialized here to simplify the example
	object = new PA_ReceiveThreadPool<SockPacket*, void*> ( this->GetObjectId() );
	paCountAlloc (object);
	object->SetConfigParameter ( "ThreadsCount", "1" );
	object->OnReconfigurationComplete();
	PA_TcpConnection::SetChildManagedObject ( "ReceiveThreadPool", object );


	object = new PA_SendThreadPool<PA_InternalPdu*, SockPacket*> ( this->GetObjectId() );
	paCountAlloc (object);
	object->SetConfigParameter ( "ThreadsCount", "1" );
	object->OnReconfigurationComplete();
	PA_TcpConnection::SetChildManagedObject ( "SendThreadPool", object );

	object = mDialogIdManager = new PA_LongRangeDialogIdManager();
	paCountAlloc (object);
	object->SetConfigParameter ( "MinDialogId", "1" );
	object->SetConfigParameter ( "MaxDialogId", "100000000" );
	object->SetConfigParameter ( "MaxInUseDialogs", "1000000" );
	object->SetConfigParameter ( "TimeOutInSeconds", "1000" );
	object->OnReconfigurationComplete();
	PA_TcpConnection::SetChildManagedObject ( "DialogIdManager", object );

	PA_TcpConnection::SetConfigParameter ( "MaxSendWaitingListSize", "1000" );
	PA_TcpConnection::SetConfigParameter ( "MaxPacketDecodePerReceive", "1" );
	PA_TcpConnection::SetConfigParameter ( "ForceSendWaitTimeInMicroSeconds", "" );
	PA_TcpConnection::SetConfigParameter ( "SendBufferSize", "" );
	PA_TcpConnection::SetConfigParameter ( "ReceiveBufferSize", "" );
}

WR_Connection::~WR_Connection()
{
	cout << BLACK "The following instance of WR_Connection is being destructed:" << endl;
	cout << RED "DN = "     << BLACK <<GetObjectDistinguishedName() << endl ;
	cout << RED "Oid = " << BLACK << GetObjectId() <<RESET_COLOR << endl;	
}

int WR_Connection::SetConfigParameter ( const string& name, const string& value )
{
	int r = 0;

	if ( name == "ResponseTimeout" )
	{
		mDialogIdManager->SetConfigParameter ( "TimeOutInSeconds", value );
	}
	else
	{
		r = PA_SmppConnection::SetConfigParameter ( name, value );
	}
	return r;
}


PDU* WR_Connection::CreatePdu ( UInteger packetId )
{
	PDU* pdu = NULL;

	switch ( packetId )
	{
		case ESME_BIND_RECEIVER:
		case ESME_BIND_TRANSMITTER:
		case ESME_BIND_TRANSCEIVER:
			pdu = new WR_Bind();
			paCountAlloc (pdu);
			break;

		case ESME_SUBMIT_SM:
			pdu = new WR_Submit();
			paCountAlloc (pdu);
			break;
		default:
			pdu = PA_SmppConnection::CreatePdu ( packetId );
	}

	return pdu;
}


ResponsePDU* WR_Connection::GenerateResponsePdu ( unsigned int originalPduType, unsigned int seqNo, int err )
{
	ResponsePDU* pdu = NULL;

	switch ( originalPduType )
	{
		case ESME_BIND_RECEIVER:
		case ESME_BIND_TRANSMITTER:
		case ESME_BIND_TRANSCEIVER:
			{
				SmppBindResp* smppBindResp = new SmppBindResp();
				paCountAlloc (smppBindResp);
				smppBindResp->Initialize ( seqNo, GetSystemId(), SmppError ( err ), BIND_TRANSCEIVER, GetInterfaceVersion() );
				pdu = smppBindResp;
			}
			break;

		default:
			pdu = PA_SmppConnection::GenerateResponsePdu ( originalPduType, seqNo, err );
	}

	return pdu;
}

void WR_Connection::SetWordReplacerOid ( ManagedObjectId oid )
{
	mWordReplacerOid = oid;


}

ManagedObjectId WR_Connection::GetWordReplacerOid()
{
	return mWordReplacerOid;
}

UInteger WR_Connection::GetExternalClientOid()
{
	return mExternalClientOid;
}

void	WR_Connection::SetExternalClientOid ( ManagedObjectId oid )
{
	mExternalClientOid = oid;
}

int WR_Connection::OnReconfigurationComplete()
{

	cout << BLACK "A new instance of WR_Connection is being" BLUE" constructed:" << endl;
	cout << BLUE "DN = "     << BLACK <<GetObjectDistinguishedName() << endl ;
	cout << BLUE "Oid = " << BLACK << GetObjectId() <<RESET_COLOR << endl;

	return PA_SmppConnection::OnReconfigurationComplete();
}


