#include "WR_ExternalClient.h"
#include "WR_Connection.h"

WR_ExternalClient::WR_ExternalClient()
{
	mReceivedPackets.SetMonitoringName ("ReceivedPackets");
	mLastMinuteStart = time (NULL);
	pthread_mutex_init (&mMutex, NULL);
	mUsageCount = 0;
}

WR_ExternalClient::~WR_ExternalClient()
{
	pthread_mutex_destroy (&mMutex);
}

int WR_ExternalClient::SetConfigParameter ( const string& name, const string& value )
{
	int r = 0;

	if ( name == "WordReplacementLicensePerMinute" )
	{
		mWordReplacementLicensePerMinute = atoi (value.c_str());
	}
	else
	{
		r = PA_SmppExternalClient::SetConfigParameter (name, value);
	}
	return r;
}

IManagedObject* WR_ExternalClient::CreateManagedObject ( const string& nodeName )
{
	IManagedObject* obj = NULL;
	if ( nodeName == "Connection" )
	{
		WR_Connection* wrc = new WR_Connection();
		paCountAlloc (wrc);
		wrc->SetWordReplacerOid ( mWordReplacerOid );
		wrc->SetExternalClientOid ( this->GetObjectId() );
		obj = wrc;
	}
	else
	{
		obj = PA_SmppExternalClient::CreateManagedObject ( nodeName );
	}
	return obj;
}

void WR_ExternalClient::SetWordReplacerOid ( ManagedObjectId oid )
{
	mWordReplacerOid = oid;
}

time_t  WR_ExternalClient::CheckLicense ( )
{
	time_t	r;
	time_t	now;

	now = time (NULL);
	pthread_mutex_lock (&mMutex);
	if (now > mLastMinuteStart+60)
	{
		mLastMinuteStart = now;
		mUsageCount = 0;
		r = 0;
	}
	else 
	{
		if (mUsageCount++ < mWordReplacementLicensePerMinute)
			r = 0;
		else
			r = mLastMinuteStart + 60 - now;
	}
	pthread_mutex_unlock (&mMutex);
	return r;
}

void WR_ExternalClient::GetMonitoringVariables ( vector<MonitoringVariable>& mvl )
{
	mvl.push_back (mReceivedPackets);
}

