#include "WordReplacer.h"
#include "WordReplacementRule.h"

WordReplacer::WordReplacer()
{
	mWordReplacementRulesList = NULL;
	mReplacedWordsCount.SetMonitoringName ("ReplacedWordsCount");
}

WordReplacer::~WordReplacer()
{
}

IManagedObject* WordReplacer::CreateManagedObject ( const string& nodeName )
{
	IManagedObject* obj = NULL;

	if ( nodeName == "WordReplacerServer" )
	{
		mWordReplacerServer = new WR_Server();
		mWordReplacerServer->SetWordReplacerOid ( this->GetObjectId() );
		obj = mWordReplacerServer;
	}
	else if ( nodeName ==  "WordReplacementRulesList" )
	{
		mWordReplacementRulesList = new WordReplacementRulesList();
		obj = mWordReplacementRulesList;
	}
	else
	{
		obj = SimpleManagedObject::CreateManagedObject ( nodeName );
	}

	return obj;
}

void WordReplacer::SetChildManagedObject ( const string& nodeName, IManagedObject* object )
{
}

string WordReplacer::ReplaceWord ( const string& origWord )
{
	string newWord;
	WordReplacementRule*	rule;
	SmartPtr<SimpleManagedObject>	sp;

	sp = mWordReplacementRulesList->FindChild ( origWord );
	rule = ( WordReplacementRule* ) sp.GetData();
	if ( rule )
	{
		newWord = rule->GetNewWord ();
		mReplacedWordsCount++;
		rule->mMatchCount++;
	}
	else
	{
		newWord = origWord;
	}

	return newWord;
}

void WordReplacer::GetMonitoringVariables ( vector<MonitoringVariable>& mvl )
{
	mvl.push_back (mReplacedWordsCount);
}

int WordReplacer::RunAction ( const string& actionName, const map<string, string>& params, vector<string>& result )
{
	int r = 0;
	map<string, string>::const_iterator itr;

	if (actionName == "ReplaceWord")
	{
		// Since action is validated with XSD, we don't check validity of parameters here.
		string s, txt, newTxt;
		itr = params.find ("Word");
		txt  = itr->second;

		for (unsigned int i = 0; i < txt.size(); i++)
		{
			string word, word2;
			for (;i < txt.size() && txt[i] != '\n' && txt[i] != ' ';i++)
			{
				word += txt[i];
			}

			word2  = ReplaceWord ( word );

			newTxt  += word2;
			newTxt  += ' ';
		}

		result.push_back (newTxt);
	}
	else
	{
		r = SimpleManagedObject::RunAction (actionName, params, result);	
	}

	return r;
}	

